/* lbackup.c
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of lbackup
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>

#include <libxfce4util/libxfce4util.h>
#include <libxfcegui4/libxfcegui4.h>

#include "lbackup.h"

#define MARGIN 2

gui *mgui;

static void
save_conf (GtkWidget *w, gpointer data)
{
    XfceRc   *rc;
    gchar    *filename;
    gboolean  c;

    c = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mgui->backup_chk));
    mgui->backup = c ? 1 : 0;

    filename = xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
        "lbackup.rc", FALSE);
    rc = xfce_rc_simple_open (filename, FALSE);
    g_free (filename);
    if (!rc)
        return;
    xfce_rc_write_int_entry (rc, "backup", c ? 1 : 0);
    xfce_rc_close (rc);
}

static void
get_conf()
{
    XfceRc *rc;
    gchar *filename;

    filename = xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
        "lbackup.rc", FALSE);
    rc = xfce_rc_simple_open (filename, TRUE);
    g_free (filename);
    if (!rc)
    {
        mgui->backup = 1;
        return;
    }
    mgui->backup = xfce_rc_read_int_entry (rc, "backup", 1);
    xfce_rc_close (rc);
}

static void
append_to_store(gpointer data, gpointer user_data)
{
    GtkListStore *store = (GtkListStore *)user_data;
    gchar        *device = (gchar *)data;
    GtkTreeIter   iter;

    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, device, -1);
}

static void
populate_store (GtkListStore *store)
{
    GList *devices;

    devices = get_valid_devices ();
    gtk_list_store_clear (store);
    g_list_foreach (devices, (GFunc) append_to_store, store);
    g_list_free (devices);
}

static void
reload_clicked (GtkWidget *btn, gpointer data)
{
    GtkWidget *combo = (GtkWidget *)data;
    GtkTreeModel *model = gtk_combo_box_get_model (GTK_COMBO_BOX (combo));
    populate_store (GTK_LIST_STORE (model));
    gtk_combo_box_set_active(GTK_COMBO_BOX (combo), 0);
}

static void
combo_changed (GtkWidget *combo, gpointer data)
{
    gtk_widget_set_sensitive ((GtkWidget *) data,
        (gtk_combo_box_get_active( GTK_COMBO_BOX (combo)) >= 0) ? TRUE : FALSE);
}

static void
go_clicked (GtkWidget *btn, gpointer data)
{
    GtkTreeIter   iter;
    GtkTreeModel *model;
    gchar        *device = NULL;
    GtkWidget    *w = (GtkWidget *)data; /* main window */

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (mgui->combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (mgui->combo), &iter);
    gtk_tree_model_get (model, &iter, 0, &device, -1);

    run_lusm (w, mgui);
}

static void
build_gui()
{
    GtkWidget       *w, *label, *go_btn, *quit_btn, *reload_btn;
    GtkWidget       *vbox, *hbox;
    GtkWidget       *heading;
    GtkListStore    *store;
    GtkCellRenderer *renderer;

    gtk_window_set_default_icon_name ("document-save");
    w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (w), _("Linutop Backup/Restore"));
    gtk_window_set_resizable (GTK_WINDOW (w), FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (w), 6);
    vbox = gtk_vbox_new (FALSE, MARGIN);
    gtk_container_add(GTK_CONTAINER (w), vbox);
    heading = xfce_heading_new();
    xfce_heading_set_title(XFCE_HEADING (heading), _("Linutop Backup/Restore"));
    xfce_heading_set_subtitle(XFCE_HEADING (heading),
        _("Backup/Restore your Linutop system"));
    xfce_heading_set_icon_name (XFCE_HEADING (heading), "document-save");
    gtk_box_pack_start(GTK_BOX (vbox), heading, FALSE, FALSE, MARGIN);
    hbox = gtk_hbox_new (FALSE, MARGIN);
    gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, FALSE, MARGIN);
    label = gtk_label_new (_("Device: "));
    gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, MARGIN);

    store = gtk_list_store_new (1, G_TYPE_STRING);
    mgui->combo = gtk_combo_box_new_with_model(GTK_TREE_MODEL (store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (mgui->combo), renderer, TRUE);
    gtk_cell_layout_add_attribute (
        GTK_CELL_LAYOUT (mgui->combo), renderer,  "text", 0);
    gtk_box_pack_start(GTK_BOX (hbox), mgui->combo, TRUE, TRUE, MARGIN);
    reload_btn = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
    gtk_box_pack_start(GTK_BOX (hbox), reload_btn, FALSE, FALSE, MARGIN);

    label = gtk_label_new ("");
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span style=\"italic\" size=\"small\">Click the 'refresh' button "
        "to detect newly plugged devices</span>"));
    gtk_box_pack_start(GTK_BOX (vbox), label, FALSE, FALSE, MARGIN);
    gtk_widget_show (label);

    mgui->backup_chk = gtk_check_button_new_with_label (
        _("Create a full copy (including user configuration)"));
    gtk_box_pack_start(GTK_BOX (vbox), mgui->backup_chk,
        FALSE, FALSE, MARGIN);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mgui->backup_chk),
        (mgui->backup) ? TRUE : FALSE);

    hbox = gtk_hbox_new (FALSE, MARGIN);
    gtk_box_pack_start(GTK_BOX (vbox), hbox, FALSE, FALSE, MARGIN);
    quit_btn = gtk_button_new_from_stock (GTK_STOCK_QUIT);
    gtk_box_pack_end(GTK_BOX (hbox), quit_btn, FALSE, FALSE, MARGIN);
    go_btn = gtk_button_new_from_stock (GTK_STOCK_APPLY);
    gtk_box_pack_end(GTK_BOX (hbox), go_btn, FALSE, FALSE, MARGIN);

    g_signal_connect (G_OBJECT (w), "destroy", gtk_main_quit, NULL);
    g_signal_connect (G_OBJECT (quit_btn), "clicked", gtk_main_quit, NULL);
    g_signal_connect (G_OBJECT (mgui->backup_chk), "toggled",
        G_CALLBACK (save_conf), NULL);
    g_signal_connect (G_OBJECT (reload_btn), "clicked",
        G_CALLBACK (reload_clicked), mgui->combo);
    g_signal_connect (G_OBJECT (mgui->combo), "changed",
        G_CALLBACK (combo_changed), go_btn);
    g_signal_connect (G_OBJECT (go_btn), "clicked",
        G_CALLBACK (go_clicked), w);

    populate_store (store);
    gtk_combo_box_set_active(GTK_COMBO_BOX (mgui->combo), 0);

    gtk_widget_show_all (w);
}

int main(int argc, char *argv[])
{
    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
    gtk_init(&argc, &argv);

    if (getuid() != 0)
    {
        xfce_warn(_("You don't appear to be root. Exiting."));
        exit (1);
    }

    mgui = g_new0(gui, 1);
    get_conf();
    build_gui();
    gtk_main();
    g_free (mgui);

    return 0;
}
